use strict;
use Test::More tests => 17;
use Test::MockObject;

use DE_EPAGES::WebInterface::API::SequenceProtocol;

my $Protocol;
my $ClientMock = Test::MockObject->new();

## no critic(ProhibitMagicNumbers)

$ClientMock->mock('send', sub {
    my $self = shift;
    my ($Packet) = @_;
    my ($Id, $Data) = $Protocol->_splitPacket($Packet); ## no critic(ProhibitPrivateSubroutineUse)
    $self->{'Sent'}->{$Id} = $Data;
});

$ClientMock->mock('receive' => sub {
    my $self = shift;
    my ($Timeout) = @_;

    my $Sent = $self->{'Sent'};

    # echo a randomly selected packet
    my @Ids = keys(%$Sent);
    unless(scalar(@Ids)) {
        sleep($Timeout);
        return;
    }
    my $RandomKey = $Ids[rand(@Ids)];

    my $Data = $Sent->{$RandomKey};
    delete $Sent->{$RandomKey};
    return $RandomKey . $Data . ' response';

});

$ClientMock->mock('sendReceive' => sub {
    my $self = shift;
    my ($Data) = @_;
    $self->send($Data);
    return $self->receive();
});

$Protocol = DE_EPAGES::WebInterface::API::SequenceProtocol->new($ClientMock);

# send

my $IdPacketMap = {};
my $TestLimit = 15;
for (0..$TestLimit) {
    my $Payload = "Test$_";
    $Protocol->send('nonsense') for(1..100);
    my $PacketId = $Protocol->send($Payload);
    my $Response = $Protocol->receive($PacketId);
    is($Response, $Payload . ' response', "Response test $PacketId");
}

# sendReceive
$Protocol->send('nonsense') for(1..100);
is($Protocol->sendReceive('SendReceiveTest1'), 'SendReceiveTest1 response', 'sendReceive');
