use strict;
use Test::More tests => 6;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

use SOAP::Lite
  on_fault => sub {
    my $soap = shift;
    my $res = shift;
    return $res;
  },
;

## no critic qw(ProhibitLongLines ProhibitMagicNumbers)
sub Main {

    my $hSystem;
    RunOnStore( Store => GetTestSite(), Sub => sub {
        $hSystem = LoadRootObject()->get([qw(DomainName WebServerDirectory WebServerScriptNamePart)]);
    });

    my $WebserviceURL = "http://$hSystem->{DomainName}/$hSystem->{WebServerDirectory}/$hSystem->{WebServerScriptNamePart}.soap";

    my $soap = SOAP::Lite
        ->uri('urn://epages.de/WebService/HelloService/2004/04')
        ->proxy($WebserviceURL);

    my $hello = $soap->hello->result;

    ok( $hello eq 'Hello', 'Hello' );

    my $time = $soap->serverTime->result;
    ok( 24 == length $time, 'ServerTime' );

    my $echo = $soap->echo('Echo')->result;
    ok( $echo eq 'Echo', 'Echo' );

    # set authentication data
    $WebserviceURL = "http://wolo:geheim\@$hSystem->{DomainName}/$hSystem->{WebServerDirectory}/$hSystem->{WebServerScriptNamePart}.soap";
    $soap->proxy($WebserviceURL);

    my $protected = $soap->protected->result;
    ok( $protected eq 'Protected', 'Protected' );

    # set wrong authentication data
    $WebserviceURL = "http://wolo:unknown\@$hSystem->{DomainName}/$hSystem->{WebServerDirectory}/$hSystem->{WebServerScriptNamePart}.soap";
    $soap->proxy($WebserviceURL);

    is $soap->protected, undef, 'protected: no result';
    ok $soap->transport->status =~ /^401/, '401 Unauthorized';
}

sub Initial {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        ImportObjects('../Samples/Server/ActionsWebServices.xml');
    });
    SynchronizeCache();
}

sub Cleanup {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        DeleteObjects('../Samples/Server/ActionsWebServices.xml');
    });
    SynchronizeCache();
}

RunScript(Sub => sub {
    Cleanup();
    Initial();
    Main();
    Cleanup();
}, Debug=>1);
